@@ -1,5 +1,5 @@
 use crate::parser::program::ctx_accounts_ident;
-use crate::{FallbackFn, Ix, IxArg};
+use crate::{FallbackFn, Ix, IxArg, IxReturn};
 use syn::parse::{Error as ParseError, Result as ParseResult};
 use syn::spanned::Spanned;
 
@@ -23,12 +23,14 @@ pub fn parse(program_mod: &syn::ItemMod) -> ParseResult<(Vec<Ix>, Option<Fallbac
         })
         .map(|method: &syn::ItemFn| {
             let (ctx, args) = parse_args(method)?;
+            let returns = parse_return(method)?;
             let anchor_ident = ctx_accounts_ident(&ctx.raw_arg)?;
             Ok(Ix {
                 raw_method: method.clone(),
                 ident: method.sig.ident.clone(),
                 args,
                 anchor_ident,
+                returns,
             })
         })
         .collect::<ParseResult<Vec<Ix>>>()?;
@@ -91,3 +93,34 @@ pub fn parse_args(method: &syn::ItemFn) -> ParseResult<(IxArg, Vec<IxArg>)> {
 
     Ok((ctx, args))
 }
+
+pub fn parse_return(method: &syn::ItemFn) -> ParseResult<IxReturn> {
+    match method.sig.output {
+        syn::ReturnType::Type(_, ref ty) => {
+            let ty = match ty.as_ref() {
+                syn::Type::Path(ty) => ty,
+                _ => return Err(ParseError::new(ty.span(), "expected a return type")),
+            };
+            // Assume unit return by default
+            let default_generic_arg = syn::GenericArgument::Type(syn::parse_str("()").unwrap());
+            let generic_args = match &ty.path.segments.last().unwrap().arguments {
+                syn::PathArguments::AngleBracketed(params) => params.args.iter().last().unwrap(),
+                _ => &default_generic_arg,
+            };
+            let ty = match generic_args {
+                syn::GenericArgument::Type(ty) => ty.clone(),
+                _ => {
+                    return Err(ParseError::new(
+                        ty.span(),
+                        "expected generic return type to be a type",
+                    ))
+                }
+            };
+            Ok(IxReturn { ty })
+        }
+        _ => Err(ParseError::new(
+            method.sig.output.span(),
+            "expected a return type",
+        )),
+    }
+}
